#include <graph/server/gui/chart/chart.h>
#include <graph/server/gui/mainwindow.h>
#include <graph/server/gui/specdialog/specdialog.h>
#include <graph/server/plugin.h>
#include <graph/server/spec.h>
#include <graph/server/ui_mainwindow.h>
#include <QFileDialog>
#include <QMessageBox>
#include <iostream>
#include <stdexcept>

namespace Graph::GUI {
MainWindow::MainWindow(QWidget* parent) : QMainWindow(parent), ui(new Ui::MainWindow) {
  ui->setupUi(this);

  // plugins
  pluginModel = new PluginModel(this);
  ui->pluginsListView->setModel(pluginModel);

  // unload plugin
  connect(ui->pluginsListView, &QListView::doubleClicked, [&](const QModelIndex& index) {
    QMessageBox::StandardButton result = QMessageBox::question(this, "Unload plugin?", "Are you sure?", QMessageBox::Yes | QMessageBox::Cancel);

    if (result == QMessageBox::Yes) {
      pluginModel->removeRow(index.row());
      pluginModel->remove(index.row());
      ui->pluginsListView->doItemsLayout();
    }
  });
}

MainWindow::~MainWindow() {
  for (ChartWidget* chart : charts) {
    delete chart;
  }

  delete pluginModel;
}

void MainWindow::on_action_Add_triggered() {
  SpecDialog dialog;
  if (dialog.exec() == SpecDialog::Rejected)
    return;

  ChartWidget* chart = new ChartWidget(ui->graphsListWidget, dialog.getSpec());

  ui->graphsListWidget->addItem(chart);
  ui->graphsListWidget->setItemWidget(chart, chart->getView());

  for (QString title : dialog.getList()) {
    QLineSeries* series = new QLineSeries();
    series->setName(title);
    chart->getChart()->addSeries(series);

    *series << QPointF(0, 1) << QPointF(1, 2) << QPoint(2, 3);
  }
}

void MainWindow::on_action_Load_triggered() {
  QString file = QFileDialog::getOpenFileName(this, "Load plugin", ".", "Plugins (*.so *.dll *.dylib)");

  if (file.isEmpty())
    return;

  try {
    pluginModel->add(new Plugin(file.toStdString()));
  } catch (std::runtime_error e) {
    std::cout << e.what() << std::endl;
    QMessageBox::critical(this, "Error", "Failed to load plugin.", QMessageBox::StandardButton::Ignore);
  }

  int row = pluginModel->rowCount();
  pluginModel->insertRow(row);
  QModelIndex index = pluginModel->index(row);
  ui->pluginsListView->doItemsLayout();
  ui->pluginsListView->setCurrentIndex(index);
}

} // namespace Graph::GUI
